#!/usr/bin/python
"""
fldigi autotuner script
Released into the public domain by Adam Greig in 2010
Modified by Laurence Blaxter
"""
import sys
import xmlrpclib
import math
import time
import subprocess
import signal

if len(sys.argv) != 3:  # the program name and the one argument
 	# stop the program and print an error message
	sys.exit("Must provide a device path and squelch on/off(e.g. /dev/ttyUSB0 0|1)")

# The squelch functionality
packet_duration=30#maximum duration of a valid string
squelch_deactive=int(sys.argv[2])
squelch_bypass=False
inactivity=0

# Desired audio frequency (Hz)
demand_freq = 2500

#the start frequency of the hardware
start_frequency = 434077387.5

# Loop time (s)
loop_time = 0.5

#the SDR PLL step size
pll_step = 225

# Connect to fldigi
s = xmlrpclib.ServerProxy("http://localhost:7362")

# Set the dial frequency correctly
s.main.set_frequency(start_frequency)

# Frequency is the actual frequency the SDR is on
frequency=start_frequency

#Fire up the c code - fix the python sigpipe issue
signal.signal(signal.SIGPIPE, signal.SIG_DFL)
bp = subprocess.Popen(['./dump', sys.argv[1]], shell=False, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

while 1:
	if squelch_deactive or s.main.get_squelch_level()>s.modem.get_quality() or squelch_bypass:#only retune when we are squelched, or squelch bypassed
		# Read current dial frequency
		dial_freq = s.main.get_frequency()
		# Check the dial frequency to see if we need to retune the SDR - this allows manual retuning
		if abs(dial_freq-frequency)>math.ceil(pll_step/2):
			frequency=(pll_step*round((dial_freq-start_frequency)/pll_step))+start_frequency
			bp.stdin.write(str(int(frequency))+'\n')
			#s.main.set_frequency(frequency)#set the dial to the actual tuned frequency	
		# Audio frequency on the waterfall
		audio_freq = s.modem.get_carrier()
		if abs(audio_freq - demand_freq) > math.ceil(pll_step/2):
			if audio_freq > demand_freq:
				#retune by +225
				bp.stdin.write("1\n")
				frequency+=pll_step
				s.modem.set_carrier(audio_freq-pll_step)
			else:
				#retune by -225
				bp.stdin.write("-1\n")
				frequency-=pll_step
				s.modem.set_carrier(audio_freq+pll_step)
			#set the dial frequency correctly
			s.main.set_frequency(frequency)
		inactivity=0#as the frequency checks were active
	# Sleep a bit
	time.sleep(loop_time)
	bp.poll()
	if bp.returncode!=None:
		sys.exit("fldigi audio pipe closed")
	inactivity+=loop_time
	if inactivity>packet_duration:#excessive inactivity indicates that we are unsquelched between packets
		squelch_bypass=True
	if s.main.get_squelch_level()>s.modem.get_quality():#once set, the squelch bypass remains until we resquelch
		squelch_bypass=False

